// TCP Echo server
// 2010/9/7


/*

(Execute Sample)
PC side:
telnet 192.168.0.25 12345
Trying 192.168.0.25...
Connected to 192.168.0.25.
Escape character is '^]'.

mbed side:
Setup OK
mbed IP Address is 192.168.0.25
Binding..
Listening...
Listening: TCP Socket Accepted
Listening: Incoming TCP connection from 192.168.0.7
TCP Socket Readable
Received&Wrote:test text

TCP Socket Writable


*/

#include "mbed.h"
#include "EthernetNetIf.h"
#include "TCPSocket.h"
#include "Json.h"

PwmOut led1(LED1, "led1");
PwmOut led4(LED4, "led4");
PwmOut left_thrust_pin(p21);
PwmOut right_thrust_pin(p22);

// EthernetNetIf eth;
EthernetNetIf eth(
  IpAddr(192,168,10,100), //IP Address
  IpAddr(255,255,255,0), //Network Mask
  IpAddr(192,168,0,1), //Gateway
  IpAddr(192,168,0,1)  //DNS
);

#define TIMEOUT_SECONDS 5.0
#define TCP_LISTENING_PORT 12345

TCPSocket ListeningSock;
TCPSocket* pConnectedSock; // for ConnectedSock
Host client;
TCPSocketErr err;
float left_motor_thrust = 0.0;
float right_motor_thrust = 0.0;
Timer recieve_timer;
Ticker pwm_ticker;
float last_recieved_time = 0.0;

float clamp(const float & value, const float & min = 0.0, const float & max = 1.0)
{
    if(value <= min) {
        return min;
    }
    if(value >= max) {
        return max;
    }
    return value;
}

void onConnectedTCPSocketEvent(TCPSocketEvent e)
{
   switch(e)
    {
    case TCPSOCKET_CONNECTED:
        printf("TCP Socket Connected\r\n");
        break;
    case TCPSOCKET_WRITEABLE:
      //Can now write some data...
        printf("TCP Socket Writable\r\n");
        break;
    case TCPSOCKET_READABLE:
      //Can now read dome data...
        printf("TCP Socket Readable\r\n");
       // Read in any available data into the buffer
       char buff[128];
       bool succees = false;
       while ( int len = pConnectedSock->recv(buff, 128) ) {
           Json json(buff, len);
           if (!json.isValidJson()) {
                succees = false;
           }
           else {
               int left_index = json.findKeyIndexIn ("left", 0 );
               int right_index = json.findKeyIndexIn ("right", 0 );
               int left_value_index = json.findChildIndexOf(left_index, -1);
               int right_value_index = json.findChildIndexOf(right_index, -1);
               if(left_index == -1 || right_index == -1) {
                   succees = false;
               }
               else {
                   if(left_value_index > 0 && right_value_index > 0) {
                       if(json.tokenNumberValue(left_value_index, left_motor_thrust) == 0 && 
                          json.tokenNumberValue(right_value_index, right_motor_thrust) == 0) {
                              last_recieved_time = recieve_timer.read();
                              succees = true;
                       }
                   }
               }
           }
           /*
           left_motor_thrust = (unsigned char)buff[0];
           right_motor_thrust = (unsigned char)buff[1];
           */
           // And send straight back out again
           // pConnectedSock->send(buff, len);
           buff[len]=0; // make terminater
           printf("Received&Wrote:%s\r\n",buff);
       }
       if(!succees) {
           left_motor_thrust = 0.0;
           right_motor_thrust = 0.0;   
       }
       break;
    case TCPSOCKET_CONTIMEOUT:
        printf("TCP Socket Timeout\r\n");
        break;
    case TCPSOCKET_CONRST:
        printf("TCP Socket CONRST\r\n");
        break;
    case TCPSOCKET_CONABRT:
        printf("TCP Socket CONABRT\r\n");
        break;
    case TCPSOCKET_ERROR:
        printf("TCP Socket Error\r\n");
        break;
    case TCPSOCKET_DISCONNECTED:
    //Close socket...
        printf("TCP Socket Disconnected\r\n");        
        pConnectedSock->close();
        break;
    default:
        printf("DEFAULT\r\n"); 
      }
}


void onListeningTCPSocketEvent(TCPSocketEvent e)
{
    switch(e)
    {
    case TCPSOCKET_ACCEPT:
        printf("Listening: TCP Socket Accepted\r\n");
        // Accepts connection from client and gets connected socket.   
        err=ListeningSock.accept(&client, &pConnectedSock);
        if (err) {
            printf("onListeningTcpSocketEvent : Could not accept connection.\r\n");
            return; //Error in accept, discard connection
        }
        // Setup the new socket events
        pConnectedSock->setOnEvent(&onConnectedTCPSocketEvent);
        // We can find out from where the connection is coming by looking at the
        // Host parameter of the accept() method
        IpAddr clientIp = client.getIp();
        printf("Listening: Incoming TCP connection from %d.%d.%d.%d\r\n", 
           clientIp[0], clientIp[1], clientIp[2], clientIp[3]);
       break;
    // the following cases will not happen
    case TCPSOCKET_CONNECTED:
        printf("Listening: TCP Socket Connected\r\n");
        break;
    case TCPSOCKET_WRITEABLE:
        printf("Listening: TCP Socket Writable\r\n");
        break;
    case TCPSOCKET_READABLE:
        printf("Listening: TCP Socket Readable\r\n");
        break;
    case TCPSOCKET_CONTIMEOUT:
        printf("Listening: TCP Socket Timeout\r\n");
        break;
    case TCPSOCKET_CONRST:
        printf("Listening: TCP Socket CONRST\r\n");
        break;
    case TCPSOCKET_CONABRT:
        printf("Listening: TCP Socket CONABRT\r\n");
        break;
    case TCPSOCKET_ERROR:
        printf("Listening: TCP Socket Error\r\n");
        break;
    case TCPSOCKET_DISCONNECTED:
    //Close socket...
        printf("Listening: TCP Socket Disconnected\r\n");        
        ListeningSock.close();
        break;
    default:
        printf("DEFAULT\r\n"); 
     };
}

void sendPwm()
{
  if(last_recieved_time - recieve_timer.read() > TIMEOUT_SECONDS) {
    left_motor_thrust = 0.0;
    right_motor_thrust = 0.0;
  }
  led1 = 0.5 + clamp(left_motor_thrust, -1, 1) * 0.5;
  led4 = 0.5 + clamp(right_motor_thrust, -1, 1) * 0.5;
  left_thrust_pin = 0.75 + clamp(left_motor_thrust, -1, 1) * 0.2;
  right_thrust_pin = 0.75 + clamp(right_motor_thrust, -1, 1) * 0.2;
}

int main() {
  led1.period(0.002);
  led4.period(0.002);
  left_thrust_pin.period(0.002);
  right_thrust_pin.period(0.002);
  printf("\r\n");
  printf("Setting up...\r\n");
  EthernetErr ethErr = eth.setup();
  if(ethErr)
  {
    printf("Error %d in setup.\r\n", ethErr);
    return -1;
  }
  printf("Setup OK\r\n");

  IpAddr ip = eth.getIp();
  printf("mbed IP Address is %d.%d.%d.%d\r\n", ip[0], ip[1], ip[2], ip[3]);

  
  // Set the callbacks for Listening
  ListeningSock.setOnEvent(&onListeningTCPSocketEvent); 
  
  // bind and listen on TCP
  err=ListeningSock.bind(Host(IpAddr(), TCP_LISTENING_PORT));
  printf("Binding..\r\n");
  if(err)
  {
    //Deal with that error...
    printf("Binding Error\n");
  }
  err=ListeningSock.listen(); // Starts listening
  printf("Listening...\r\n");
  if(err)
  {
    printf("Listening Error\r\n");
  }
  recieve_timer.start();
  pwm_ticker.attach(&sendPwm, 0.01);
  while(true)
  {
    Net::poll();
  }  
}
